@@ -10,10 +10,9 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-import mock
 from datetime import datetime
 
-from tests import BaseSessionTest, ClientHTTPStubber
+from tests import BaseSessionTest, ClientHTTPStubber, mock
 
 
 class TestLex(BaseSessionTest):
