@@ -10,22 +10,29 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+import copy
+import os
+
 import pytest
 
-from tests import mock
-from tests import unittest
-
-import botocore
+import botocore.configprovider
 import botocore.session as session
-from botocore.configprovider import ConfigValueStore
-from botocore.configprovider import BaseProvider
-from botocore.configprovider import InstanceVarProvider
-from botocore.configprovider import EnvironmentProvider
-from botocore.configprovider import ScopedConfigProvider
-from botocore.configprovider import SectionConfigProvider
-from botocore.configprovider import ConstantProvider
-from botocore.configprovider import ChainProvider
-from botocore.configprovider import ConfigChainFactory
+from botocore.configprovider import (
+    BaseProvider,
+    ChainProvider,
+    ConfigChainFactory,
+    ConfigValueStore,
+    ConstantProvider,
+    DefaultConfigResolver,
+    EnvironmentProvider,
+    InstanceVarProvider,
+    ScopedConfigProvider,
+    SectionConfigProvider,
+    SmartDefaultsConfigStoreFactory,
+)
+from botocore.exceptions import ConnectTimeoutError
+from botocore.utils import IMDSRegionProvider
+from tests import mock, unittest
 
 
 class TestConfigChainFactory(unittest.TestCase):
@@ -328,6 +335,28 @@ class TestConfigValueStore(unittest.TestCase):
         value = provider.get_config_variable('fake_variable')
         self.assertEqual(value, 'bar')
 
+    def test_can_get_config_provider(self):
+        chain_provider = ChainProvider(
+            providers=[ConstantProvider(value='bar')]
+        )
+        config_value_store = ConfigValueStore(mapping={
+            'fake_variable': chain_provider,
+        })
+        provider = config_value_store.get_config_provider('fake_variable')
+        value = config_value_store.get_config_variable('fake_variable')
+        self.assertIsInstance(provider, ChainProvider)
+        self.assertEqual(value, 'bar')
+
+    def test_can_get_config_provider_non_chain_provider(self):
+        constant_provider = ConstantProvider(value='bar')
+        config_value_store = ConfigValueStore(mapping={
+            'fake_variable': constant_provider,
+        })
+        provider = config_value_store.get_config_provider('fake_variable')
+        value = config_value_store.get_config_variable('fake_variable')
+        self.assertIsInstance(provider, ConstantProvider)
+        self.assertEqual(value, 'bar')
+
 
 class TestInstanceVarProvider(unittest.TestCase):
     def assert_provides_value(self, name, instance_map, expected_value):
@@ -557,3 +586,224 @@ class TestSectionConfigProvider(unittest.TestCase):
                 'override_var': 'override',
             }
         )
+
+
+class TestSmartDefaults:
+
+    def _template(self):
+        return {
+            "base": {
+                "retryMode": "standard",
+                "stsRegionalEndpoints": "regional",
+                "s3UsEast1RegionalEndpoints": "regional",
+                "connectTimeoutInMillis": 1000,
+                "tlsNegotiationTimeoutInMillis": 1000
+            },
+            "modes": {
+                "standard": {
+                    "connectTimeoutInMillis": {
+                        "multiply": 2
+                    },
+                    "tlsNegotiationTimeoutInMillis": {
+                        "multiply": 2
+                    }
+                },
+                "in-region": {
+                    "connectTimeoutInMillis": {
+                        "multiply": 1
+                    },
+                    "tlsNegotiationTimeoutInMillis": {
+                        "multiply": 1
+                    }
+                },
+                "cross-region": {
+                    "connectTimeoutInMillis": {
+                        "multiply": 2.8
+                    },
+                    "tlsNegotiationTimeoutInMillis": {
+                        "multiply": 2.8
+                    }
+                },
+                "mobile": {
+                    "connectTimeoutInMillis": {
+                        "override": 10000
+                    },
+                    "tlsNegotiationTimeoutInMillis": {
+                        "add": 10000
+                    },
+                    "retryMode": {
+                        "override": "adaptive"
+                    }
+                }
+            }
+        }
+
+    def _create_default_config_resolver(self):
+        return DefaultConfigResolver(self._template())
+
+    @pytest.fixture
+    def smart_defaults_factory(self):
+        fake_session = mock.Mock(spec=session.Session)
+        fake_session.get_scoped_config.return_value = {}
+        default_config_resolver = self._create_default_config_resolver()
+        return SmartDefaultsConfigStoreFactory(
+            default_config_resolver, imds_region_provider=mock.Mock()
+        )
+
+    @pytest.fixture
+    def fake_session(self):
+        fake_session = mock.Mock(spec=session.Session)
+        fake_session.get_scoped_config.return_value = {}
+        return fake_session
+
+    def _create_config_value_store(self, s3_mapping={}, **override_kwargs):
+        provider_foo = ConstantProvider(value='foo')
+        environment_provider_foo = EnvironmentProvider(
+            name='AWS_RETRY_MODE',
+            env={'AWS_RETRY_MODE': None}
+        )
+        fake_session = mock.Mock(spec=session.Session)
+        fake_session.get_scoped_config.return_value = {}
+        # Testing with three different providers to validate
+        # SmartDefaultsConfigStoreFactory._get_new_chain_provider
+        mapping = {
+            'sts_regional_endpoints': ChainProvider(providers=[provider_foo]),
+            'retry_mode': ChainProvider(providers=[environment_provider_foo]),
+            's3': SectionConfigProvider('s3', fake_session, s3_mapping)
+        }
+        mapping.update(**override_kwargs)
+        config_store = ConfigValueStore(mapping=mapping)
+        return config_store
+
+    def _create_os_environ_patcher(self):
+        return mock.patch.object(
+            botocore.configprovider.os, 'environ',
+            mock.Mock(wraps=os.environ)
+        )
+
+    def test_config_store_deepcopy(self):
+        config_store = ConfigValueStore()
+        config_store.set_config_provider('foo', ConstantProvider('bar'))
+        config_store_copy = copy.deepcopy(config_store)
+        config_store_copy.set_config_provider('fizz', ConstantProvider('buzz'))
+        assert config_store.get_config_variable('fizz') is None
+        assert config_store_copy.get_config_variable('foo') == 'bar'
+
+    @pytest.mark.parametrize(
+        'defaults_mode, retry_mode, sts_regional_endpoints,'
+        ' us_east_1_regional_endpoint, connect_timeout',
+        [
+            ('standard', 'standard', 'regional', 'regional', 2000),
+            ('in-region', 'standard', 'regional', 'regional', 1000),
+            ('cross-region', 'standard', 'regional', 'regional', 2800),
+            ('mobile', 'adaptive', 'regional', 'regional', 10000),
+        ]
+    )
+    def test_get_defualt_config_values(self, defaults_mode, retry_mode,
+                                       sts_regional_endpoints,
+                                       us_east_1_regional_endpoint,
+                                       connect_timeout):
+        default_config_resolver = self._create_default_config_resolver()
+        default_values = default_config_resolver.get_default_config_values(
+            defaults_mode
+        )
+        assert default_values['retryMode'] == retry_mode
+        assert default_values['stsRegionalEndpoints'] == sts_regional_endpoints
+        assert default_values['s3UsEast1RegionalEndpoints'] == us_east_1_regional_endpoint
+        assert default_values['connectTimeoutInMillis'] == connect_timeout
+
+    def test_resolve_default_values_on_config(self, smart_defaults_factory,
+                                              fake_session):
+        config_store = self._create_config_value_store()
+        smart_defaults_factory.merge_smart_defaults(
+            config_store, 'standard', 'foo')
+        s3_config = config_store.get_config_variable('s3')
+        assert s3_config['us_east_1_regional_endpoint'] == 'regional'
+        assert config_store.get_config_variable('retry_mode') == 'standard'
+        assert config_store.get_config_variable('sts_regional_endpoints') == 'regional'
+        assert config_store.get_config_variable('connect_timeout') == 2
+
+    def test_no_resolve_default_s3_values_on_config(self,
+                                                    smart_defaults_factory,
+                                                    fake_session):
+        environment_provider = EnvironmentProvider(
+            name='AWS_S3_US_EAST_1_REGIONAL_ENDPOINT',
+            env={'AWS_S3_US_EAST_1_REGIONAL_ENDPOINT': 'legacy'}
+        )
+        s3_mapping = {
+            'us_east_1_regional_endpoint': ChainProvider(
+                providers=[environment_provider])
+        }
+        config_store = self._create_config_value_store(s3_mapping=s3_mapping)
+        smart_defaults_factory.merge_smart_defaults(
+            config_store, 'standard', 'foo')
+        s3_config = config_store.get_config_variable('s3')
+        assert s3_config['us_east_1_regional_endpoint'] == 'legacy'
+        assert config_store.get_config_variable('retry_mode') == 'standard'
+        assert config_store.get_config_variable('sts_regional_endpoints') == 'regional'
+        assert config_store.get_config_variable('connect_timeout') == 2
+
+    def test_resolve_default_s3_values_on_config(self, smart_defaults_factory,
+                                                 fake_session):
+        s3_mapping = {
+            'use_arn_region': ChainProvider(
+                providers=[ConstantProvider(value=False)])
+        }
+        config_store = self._create_config_value_store(s3_mapping=s3_mapping)
+        smart_defaults_factory.merge_smart_defaults(
+            config_store, 'standard', 'foo')
+        s3_config = config_store.get_config_variable('s3')
+        assert s3_config['us_east_1_regional_endpoint'] == 'regional'
+        assert config_store.get_config_variable('retry_mode') == 'standard'
+        assert config_store.get_config_variable('sts_regional_endpoints') == 'regional'
+        assert config_store.get_config_variable('connect_timeout') == 2
+
+    @pytest.mark.parametrize(
+        'execution_env_var, region_env_var, default_region_env_var, '
+        'imds_region, client_region, resolved_mode',
+        [
+            ('AWS_Lambda_python3.6', 'us-east-1', None,
+             None, 'us-east-1', 'in-region'),
+            ('AWS_Lambda_python3.6', 'us-west-2', 'us-west-2',
+             None, 'us-east-1', 'cross-region'),
+            ('AWS_Lambda_python3.6', None, None,
+             'us-west-2', 'us-east-1', 'cross-region'),
+            (None, None, 'us-east-1',
+             'us-east-1', 'us-east-1', 'in-region'),
+            (None, None, None,
+             'us-west-2', 'us-east-1', 'cross-region'),
+            (None, None, None,
+             None, 'us-west-2', 'standard'),
+        ]
+    )
+    def test_resolve_auto_mode(self, execution_env_var, region_env_var,
+                               default_region_env_var,
+                               imds_region, client_region, resolved_mode):
+        imds_region_provider = mock.Mock(spec=IMDSRegionProvider)
+        imds_region_provider.provide.return_value = imds_region
+        default_config_resolver = mock.Mock()
+        with mock.patch.object(
+            botocore.configprovider.os, 'environ',
+            mock.Mock(wraps=os.environ)
+        ) as os_environ_patcher:
+            os_environ_patcher.get.side_effect = [
+                execution_env_var, default_region_env_var, region_env_var]
+            smart_defaults_factory = SmartDefaultsConfigStoreFactory(
+                default_config_resolver, imds_region_provider)
+            mode = smart_defaults_factory.resolve_auto_mode(client_region)
+            assert mode == resolved_mode
+
+    def test_resolve_auto_mode_imds_region_provider_connect_timeout(self):
+        imds_region_provider = mock.Mock(spec=IMDSRegionProvider)
+        imds_region_provider.provide.side_effect = ConnectTimeoutError(
+            endpoint_url='foo')
+        default_config_resolver = mock.Mock()
+        with mock.patch.object(
+            botocore.configprovider.os, 'environ',
+            mock.Mock(wraps=os.environ)
+        ) as os_environ_patcher:
+            os_environ_patcher.get.side_effect = [None] * 3
+            smart_defaults_factory = SmartDefaultsConfigStoreFactory(
+                default_config_resolver, imds_region_provider)
+            mode = smart_defaults_factory.resolve_auto_mode('us-west-2')
+            assert mode == 'standard'
